@@ -59,6 +59,8 @@ import org.omg.IOP.TaggedProfileHelper;
 import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.zip.Adler32;
 
 /**
  * The implementaton of the Interoperable Object Reference (IOR). IOR can be
@@ -138,6 +140,27 @@ public class IOR
         b.append(' ');
         return b.toString();
       }
+      
+      /**
+       * Get a better formatted multiline string representation.
+       */
+      public String toStringFormatted()
+      {
+        StringBuffer b = new StringBuffer();
+        b.append("\n  Native set " + name(native_set));
+        if (conversion != null && conversion.length > 0)
+          {
+            b.append("\n  Other supported sets:\n    ");
+            for (int i = 0; i < conversion.length; i++)
+              {
+                b.append(name(conversion[i]));
+                b.append(' ');
+              }
+          }
+        b.append("\n");
+        return b.toString();
+      }
+      
 
       /**
        * Write into CDR stream.
@@ -362,7 +385,7 @@ public class IOR
   /**
    * Parse the provided stringifed reference.
    *
-   * @param stringified_reference, in the form of IOR:nnnnnn.....
+   * @param stringified_reference in the form of IOR:nnnnnn.....
    *
    * @return the parsed IOR
    *
@@ -588,6 +611,39 @@ public class IOR
 
     return b.toString();
   }
+  
+  /**
+   * Returns a multiline formatted human readable string representation of 
+   * this IOR object.
+   */
+  public String toStringFormatted()
+  {
+    StringBuffer b = new StringBuffer();
+    b.append("\nObject Id:\n  ");
+    b.append(Id);
+    b.append("\nObject is accessible at:\n  ");
+    b.append(Internet);
+
+    if (Big_Endian)
+      b.append("\n  Big endian encoding");
+    else
+      b.append("\n  Little endian encoding.");      
+
+    b.append("\nObject Key\n  ");
+
+    for (int i = 0; i < key.length; i++)
+      {
+        b.append(Integer.toHexString(key[i] & 0xFF));
+      }
+
+    b.append("\nSupported code sets:");
+    b.append("\n Wide:");
+    b.append(Internet.CodeSets.wide.toStringFormatted());
+    b.append(" Narrow:");
+    b.append(Internet.CodeSets.wide.toStringFormatted());
+
+    return b.toString();
+  }  
 
   /**
    * Returs a stringified reference.
@@ -716,4 +772,51 @@ public class IOR
       // The future supported tagged profiles should be added here.
       throw new BAD_PARAM("Unsupported profile type " + profile.tag);
   }
+  
+  /**
+   * Checks for equality.
+   */
+  public boolean equals(Object x)
+  {
+    if (x instanceof IOR)
+      {
+        boolean keys;
+        boolean hosts = true;
+
+        IOR other = (IOR) x;
+        
+        if (Internet==null || other.Internet==null)
+          return Internet == other.Internet;
+        
+        if (key != null && other.key != null)
+          keys = Arrays.equals(key, other.key);
+        else
+          keys = key == other.key;
+
+        if (Internet != null && Internet.host != null)
+          if (other.Internet != null && other.Internet.host != null)
+            hosts = other.Internet.host.equals(Internet.host);
+
+        return keys & hosts && Internet.port==other.Internet.port;
+      }
+    else
+      return false;
+  }
+  
+  /**
+   * Get the hashcode of this IOR.
+   */
+  public int hashCode()
+  {
+    Adler32 adler = new Adler32();
+    if (key != null)
+      adler.update(key);
+    if (Internet != null)
+      {
+        if (Internet.host != null)
+          adler.update(Internet.host.getBytes());
+        adler.update(Internet.port);
+      }
+    return (int) adler.getValue();
+  }
 }
\ No newline at end of file
