@@ -38,13 +38,12 @@ exception statement from your version. */
 
 package gnu.java.nio.charset.iconv;
 
-import gnu.classpath.RawData;
+import gnu.classpath.Pointer;
 
 import java.nio.ByteBuffer;
 import java.nio.CharBuffer;
 import java.nio.charset.Charset;
 import java.nio.charset.CharsetDecoder;
-import java.nio.charset.CharsetEncoder;
 import java.nio.charset.CoderResult;
 
 final class IconvDecoder extends CharsetDecoder
@@ -55,7 +54,7 @@ final class IconvDecoder extends CharsetDecoder
     openIconv(info.iconvName());
   }
 
-  private RawData data;
+  private Pointer data;
   private int inremaining;
   private int outremaining;
 
