@@ -73,7 +73,7 @@ public final class CollationElementIterator
   /**
    * This is the String that is being iterated over.
    */
-  String text;
+  CharacterIterator text;
 
   /**
    * This is the index into the collation decomposition where we are currently scanning.
@@ -111,6 +111,21 @@ public final class CollationElementIterator
     setText (text);    
   }
 
+  /**
+   * This method initializes a new instance of <code>CollationElementIterator</code>
+   * to iterate over the specified <code>String</code> using the rules in the
+   * specified <code>RuleBasedCollator</code>.
+   *
+   * @param collator The <code>RuleBasedCollation</code> used for calculating collation values
+   * @param text The character iterator to iterate over.
+   */
+  CollationElementIterator(RuleBasedCollator collator, CharacterIterator text)
+  {
+    this.collator = collator;
+    
+    setText (text);    
+  }
+
   RuleBasedCollator.CollationElement nextBlock()
   {
     if (index >= text_decomposition.length)
@@ -177,9 +192,11 @@ public final class CollationElementIterator
    * This method returns the primary order value for the given collation
    * value.
    *
-   * @param value The collation value returned from <code>next()</code> or <code>previous()</code>.
+   * @param order The collation value returned from <code>next()</code> or 
+   *              <code>previous()</code>.
    *
-   * @return The primary order value of the specified collation value.  This is the high 16 bits.
+   * @return The primary order value of the specified collation value.  This is
+   *         the high 16 bits.
    */
   public static int primaryOrder(int order)
   {
@@ -201,9 +218,11 @@ public final class CollationElementIterator
    * This method returns the secondary order value for the given collation
    * value.
    *
-   * @param value The collation value returned from <code>next()</code> or <code>previous()</code>.
+   * @param order The collation value returned from <code>next()</code> or 
+   *              <code>previous()</code>.
    *
-   * @return The secondary order value of the specified collation value.  This is the bits 8-15.
+   * @return The secondary order value of the specified collation value.  This 
+   *         is the bits 8-15.
    */
   public static short secondaryOrder(int order)
   {
@@ -215,9 +234,11 @@ public final class CollationElementIterator
    * This method returns the tertiary order value for the given collation
    * value.
    *
-   * @param value The collation value returned from <code>next()</code> or <code>previous()</code>.
+   * @param order The collation value returned from <code>next()</code> or 
+   *              <code>previous()</code>.
    *
-   * @return The tertiary order value of the specified collation value.  This is the low eight bits.
+   * @return The tertiary order value of the specified collation value.  This 
+   *         is the low eight bits.
    */
   public static short tertiaryOrder(int order)
   {
@@ -240,7 +261,7 @@ public final class CollationElementIterator
     int alreadyExpanded = 0;
     int idxToMove = 0;
 
-    this.text = text;
+    this.text = new StringCharacterIterator(text);
     this.index = 0;
 
     String work_text = text.intern();
@@ -434,7 +455,7 @@ public final class CollationElementIterator
     if (offset < 0)
       throw new IllegalArgumentException("Negative offset: " + offset);
 
-    if (offset > (text.length() - 1))
+    if (offset > (text.getEndIndex() - 1))
       throw new IllegalArgumentException("Offset too large: " + offset);
     
     for (index = 0; index < text_decomposition.length; index++)
@@ -458,7 +479,7 @@ public final class CollationElementIterator
    *
    * @param value The collation order value
    *
-   * @param The maximum length of an expansion sequence.
+   * @return The maximum length of an expansion sequence.
    */
   public int getMaxExpansion(int value)
   {
