@@ -139,9 +139,9 @@ public class SimpleDateFormat extends DateFormat
      */
     public String toString()
     {
-      StringBuffer builder;
+      StringBuilder builder;
 
-      builder = new StringBuffer(getClass().getName());
+      builder = new StringBuilder(getClass().getName());
       builder.append("[field=");
       builder.append(field);
       builder.append(", size=");
@@ -307,7 +307,7 @@ public class SimpleDateFormat extends DateFormat
 		// Not a valid letter
 		throw new IllegalArgumentException("Invalid letter "
 						   + thisChar +
-						   "encountered at character "
+						   " encountered at character "
 						   + i + ".");
 	      }
 	    else if (thisChar == '\'')
@@ -322,7 +322,7 @@ public class SimpleDateFormat extends DateFormat
 		    // Look for the terminating quote.  However, if we
 		    // see a '', that represents a literal quote and
 		    // we must iterate.
-		    StringBuffer buf = new StringBuffer();
+		    StringBuilder buf = new StringBuilder();
 		    int oldPos = i + 1;
 		    do
 		      {
@@ -346,7 +346,7 @@ public class SimpleDateFormat extends DateFormat
 	    else
 	      {
 		// A special character
-		tokens.add(new Character(thisChar));
+		tokens.add(Character.valueOf(thisChar));
 	      }
 	  }
 	else
@@ -372,7 +372,7 @@ public class SimpleDateFormat extends DateFormat
    */
   public String toString() 
   {
-    StringBuffer output = new StringBuffer(getClass().getName());
+    StringBuilder output = new StringBuilder(getClass().getName());
     output.append("[tokens=");
     output.append(tokens);
     output.append(", formatData=");
@@ -554,7 +554,7 @@ public class SimpleDateFormat extends DateFormat
 					   String oldChars, String newChars)
   {
     int len = pattern.length();
-    StringBuffer buf = new StringBuffer(len);
+    StringBuilder buf = new StringBuilder(len);
     boolean quoted = false;
     for (int i = 0;  i < len;  i++)
       {
@@ -805,7 +805,8 @@ public class SimpleDateFormat extends DateFormat
 		buffer.setDefaultAttribute(DateFormat.Field.RFC822_TIME_ZONE);
 		int pureMinutes = (calendar.get(Calendar.ZONE_OFFSET) +
 				   calendar.get(Calendar.DST_OFFSET)) / (1000 * 60);
-		String sign = (pureMinutes < 0) ? "-" : "+";	  
+		String sign = (pureMinutes < 0) ? "-" : "+";
+                pureMinutes = Math.abs(pureMinutes);
 		int hours = pureMinutes / 60;
 		int minutes = pureMinutes % 60;
 		buffer.append(sign);
@@ -916,7 +917,25 @@ public class SimpleDateFormat extends DateFormat
 		|| ((ch < 'a' || ch > 'z')
 		    && (ch < 'A' || ch > 'Z')))
 	      {
-		if (! expect (dateStr, pos, ch))
+                if (quote_start == -1 && ch == ' ')
+                  {
+                    // A single unquoted space in the pattern may match
+                    // any number of spaces in the input.
+                    int index = pos.getIndex();
+                    int save = index;
+                    while (index < dateStr.length()
+                           && Character.isWhitespace(dateStr.charAt(index)))
+                      ++index;
+                    if (index > save)
+                      pos.setIndex(index);
+                    else
+                      {
+                        // Didn't see any whitespace.
+                        pos.setErrorIndex(index);
+                        return null;
+                      }
+                  }
+                else if (! expect (dateStr, pos, ch))
 		  return null;
 		continue;
 	      }
@@ -1082,11 +1101,21 @@ public class SimpleDateFormat extends DateFormat
 	    if (is_numeric)
 	      {
 		numberFormat.setMinimumIntegerDigits(fmt_count);
-		if (limit_digits)
-		  numberFormat.setMaximumIntegerDigits(fmt_count);
 		if (maybe2DigitYear)
 		  index = pos.getIndex();
-		Number n = numberFormat.parse(dateStr, pos);
+		Number n = null;
+		if (limit_digits)
+		  {
+		    // numberFormat.setMaximumIntegerDigits(fmt_count) may
+		    // not work as expected. So we explicitly use substring
+		    // of dateStr.
+		    int origPos = pos.getIndex();
+		    pos.setIndex(0);
+		    n = numberFormat.parse(dateStr.substring(origPos, origPos + fmt_count), pos);
+		    pos.setIndex(origPos + pos.getIndex());
+		  }
+		else
+		  n = numberFormat.parse(dateStr, pos);
 		if (pos == null || ! (n instanceof Long))
 		  return null;
 		value = n.intValue() + offset;
@@ -1250,12 +1279,12 @@ public class SimpleDateFormat extends DateFormat
 
 	// advance the index
 	pos.setIndex(pos.getIndex() + matcher.end());
-	return new Integer(offset);
+	return Integer.valueOf(offset);
       }
     else if (zoneString.startsWith("GMT"))
       {
 	pos.setIndex(pos.getIndex() + 3);
-	return new Integer(0);
+	return Integer.valueOf(0);
       }
     return null;
   }
