@@ -1,5 +1,5 @@
 /* ULongSeqHelper.java --
-   Copyright (C) 2005 Free Software Foundation, Inc.
+   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -38,11 +38,12 @@ exception statement from your version. */
 
 package org.omg.CORBA;
 
-import gnu.CORBA.primitiveArrayTypeCode;
+import gnu.CORBA.typecodes.ArrayTypeCode;
 
 import org.omg.CORBA.TypeCodePackage.BadKind;
 import org.omg.CORBA.portable.InputStream;
 import org.omg.CORBA.portable.OutputStream;
+import org.omg.CORBA.portable.Streamable;
 
 /**
  * Provides static helper methods for working with
@@ -125,7 +126,7 @@ public abstract class ULongSeqHelper
    */
   public static TypeCode type()
   {
-    return new primitiveArrayTypeCode(TCKind.tk_long);
+    return new ArrayTypeCode(TCKind.tk_long);
   }
 
   /**
